@@ -14,8 +14,8 @@
 from __future__ import annotations
 
 from bigframes.core.compile.api import test_only_ibis_inferred_schema
-from bigframes.core.compile.compiler import compile_sql
 from bigframes.core.compile.configs import CompileRequest, CompileResult
+from bigframes.core.compile.ibis_compiler.ibis_compiler import compile_sql
 
 __all__ = [
     "test_only_ibis_inferred_schema",
