@@ -38,21 +38,15 @@ def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
         return sge.Concat(expressions=[left.expr, right.expr])
 
     if dtypes.is_numeric(left.dtype) and dtypes.is_numeric(right.dtype):
-        left_expr = left.expr
-        if left.dtype == dtypes.BOOL_DTYPE:
-            left_expr = sge.Cast(this=left_expr, to="INT64")
-        right_expr = right.expr
-        if right.dtype == dtypes.BOOL_DTYPE:
-            right_expr = sge.Cast(this=right_expr, to="INT64")
+        left_expr = _coerce_bool_to_int(left)
+        right_expr = _coerce_bool_to_int(right)
         return sge.Add(this=left_expr, expression=right_expr)
 
     if (
         dtypes.is_time_or_date_like(left.dtype)
         and right.dtype == dtypes.TIMEDELTA_DTYPE
     ):
-        left_expr = left.expr
-        if left.dtype == dtypes.DATE_DTYPE:
-            left_expr = sge.Cast(this=left_expr, to="DATETIME")
+        left_expr = _coerce_date_to_datetime(left)
         return sge.TimestampAdd(
             this=left_expr, expression=right.expr, unit=sge.Var(this="MICROSECOND")
         )
@@ -60,9 +54,7 @@ def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
         dtypes.is_time_or_date_like(right.dtype)
         and left.dtype == dtypes.TIMEDELTA_DTYPE
     ):
-        right_expr = right.expr
-        if right.dtype == dtypes.DATE_DTYPE:
-            right_expr = sge.Cast(this=right_expr, to="DATETIME")
+        right_expr = _coerce_date_to_datetime(right)
         return sge.TimestampAdd(
             this=right_expr, expression=left.expr, unit=sge.Var(this="MICROSECOND")
         )
@@ -74,14 +66,37 @@ def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
     )
 
 
-@BINARY_OP_REGISTRATION.register(ops.div_op)
+@BINARY_OP_REGISTRATION.register(ops.eq_op)
+def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
+    left_expr = _coerce_bool_to_int(left)
+    right_expr = _coerce_bool_to_int(right)
+    return sge.EQ(this=left_expr, expression=right_expr)
+
+
+@BINARY_OP_REGISTRATION.register(ops.eq_null_match_op)
 def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
     left_expr = left.expr
-    if left.dtype == dtypes.BOOL_DTYPE:
-        left_expr = sge.Cast(this=left_expr, to="INT64")
+    if right.dtype != dtypes.BOOL_DTYPE:
+        left_expr = _coerce_bool_to_int(left)
+
     right_expr = right.expr
-    if right.dtype == dtypes.BOOL_DTYPE:
-        right_expr = sge.Cast(this=right_expr, to="INT64")
+    if left.dtype != dtypes.BOOL_DTYPE:
+        right_expr = _coerce_bool_to_int(right)
+
+    sentinel = sge.convert("$NULL_SENTINEL$")
+    left_coalesce = sge.Coalesce(
+        this=sge.Cast(this=left_expr, to="STRING"), expressions=[sentinel]
+    )
+    right_coalesce = sge.Coalesce(
+        this=sge.Cast(this=right_expr, to="STRING"), expressions=[sentinel]
+    )
+    return sge.EQ(this=left_coalesce, expression=right_coalesce)
+
+
+@BINARY_OP_REGISTRATION.register(ops.div_op)
+def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
+    left_expr = _coerce_bool_to_int(left)
+    right_expr = _coerce_bool_to_int(right)
 
     result = sge.func("IEEE_DIVIDE", left_expr, right_expr)
     if left.dtype == dtypes.TIMEDELTA_DTYPE and dtypes.is_numeric(right.dtype):
@@ -92,12 +107,8 @@ def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
 
 @BINARY_OP_REGISTRATION.register(ops.floordiv_op)
 def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
-    left_expr = left.expr
-    if left.dtype == dtypes.BOOL_DTYPE:
-        left_expr = sge.Cast(this=left_expr, to="INT64")
-    right_expr = right.expr
-    if right.dtype == dtypes.BOOL_DTYPE:
-        right_expr = sge.Cast(this=right_expr, to="INT64")
+    left_expr = _coerce_bool_to_int(left)
+    right_expr = _coerce_bool_to_int(right)
 
     result: sge.Expression = sge.Cast(
         this=sge.Floor(this=sge.func("IEEE_DIVIDE", left_expr, right_expr)), to="INT64"
@@ -129,7 +140,16 @@ def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
 
 @BINARY_OP_REGISTRATION.register(ops.ge_op)
 def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
-    return sge.GTE(this=left.expr, expression=right.expr)
+    left_expr = _coerce_bool_to_int(left)
+    right_expr = _coerce_bool_to_int(right)
+    return sge.GTE(this=left_expr, expression=right_expr)
+
+
+@BINARY_OP_REGISTRATION.register(ops.gt_op)
+def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
+    left_expr = _coerce_bool_to_int(left)
+    right_expr = _coerce_bool_to_int(right)
+    return sge.GT(this=left_expr, expression=right_expr)
 
 
 @BINARY_OP_REGISTRATION.register(ops.JSONSet)
@@ -137,14 +157,24 @@ def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
     return sge.func("JSON_SET", left.expr, sge.convert(op.json_path), right.expr)
 
 
+@BINARY_OP_REGISTRATION.register(ops.lt_op)
+def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
+    left_expr = _coerce_bool_to_int(left)
+    right_expr = _coerce_bool_to_int(right)
+    return sge.LT(this=left_expr, expression=right_expr)
+
+
+@BINARY_OP_REGISTRATION.register(ops.le_op)
+def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
+    left_expr = _coerce_bool_to_int(left)
+    right_expr = _coerce_bool_to_int(right)
+    return sge.LTE(this=left_expr, expression=right_expr)
+
+
 @BINARY_OP_REGISTRATION.register(ops.mul_op)
 def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
-    left_expr = left.expr
-    if left.dtype == dtypes.BOOL_DTYPE:
-        left_expr = sge.Cast(this=left_expr, to="INT64")
-    right_expr = right.expr
-    if right.dtype == dtypes.BOOL_DTYPE:
-        right_expr = sge.Cast(this=right_expr, to="INT64")
+    left_expr = _coerce_bool_to_int(left)
+    right_expr = _coerce_bool_to_int(right)
 
     result = sge.Mul(this=left_expr, expression=right_expr)
 
@@ -156,36 +186,38 @@ def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
         return result
 
 
+@BINARY_OP_REGISTRATION.register(ops.ne_op)
+def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
+    left_expr = _coerce_bool_to_int(left)
+    right_expr = _coerce_bool_to_int(right)
+    return sge.NEQ(this=left_expr, expression=right_expr)
+
+
+@BINARY_OP_REGISTRATION.register(ops.obj_make_ref_op)
+def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
+    return sge.func("OBJ.MAKE_REF", left.expr, right.expr)
+
+
 @BINARY_OP_REGISTRATION.register(ops.sub_op)
 def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
     if dtypes.is_numeric(left.dtype) and dtypes.is_numeric(right.dtype):
-        left_expr = left.expr
-        if left.dtype == dtypes.BOOL_DTYPE:
-            left_expr = sge.Cast(this=left_expr, to="INT64")
-        right_expr = right.expr
-        if right.dtype == dtypes.BOOL_DTYPE:
-            right_expr = sge.Cast(this=right_expr, to="INT64")
+        left_expr = _coerce_bool_to_int(left)
+        right_expr = _coerce_bool_to_int(right)
         return sge.Sub(this=left_expr, expression=right_expr)
 
     if (
         dtypes.is_time_or_date_like(left.dtype)
         and right.dtype == dtypes.TIMEDELTA_DTYPE
     ):
-        left_expr = left.expr
-        if left.dtype == dtypes.DATE_DTYPE:
-            left_expr = sge.Cast(this=left_expr, to="DATETIME")
+        left_expr = _coerce_date_to_datetime(left)
         return sge.TimestampSub(
             this=left_expr, expression=right.expr, unit=sge.Var(this="MICROSECOND")
         )
     if dtypes.is_time_or_date_like(left.dtype) and dtypes.is_time_or_date_like(
         right.dtype
     ):
-        left_expr = left.expr
-        if left.dtype == dtypes.DATE_DTYPE:
-            left_expr = sge.Cast(this=left_expr, to="DATETIME")
-        right_expr = right.expr
-        if right.dtype == dtypes.DATE_DTYPE:
-            right_expr = sge.Cast(this=right_expr, to="DATETIME")
+        left_expr = _coerce_date_to_datetime(left)
+        right_expr = _coerce_date_to_datetime(right)
         return sge.TimestampDiff(
             this=left_expr, expression=right_expr, unit=sge.Var(this="MICROSECOND")
         )
@@ -198,6 +230,15 @@ def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
     )
 
 
-@BINARY_OP_REGISTRATION.register(ops.obj_make_ref_op)
-def _(op, left: TypedExpr, right: TypedExpr) -> sge.Expression:
-    return sge.func("OBJ.MAKE_REF", left.expr, right.expr)
+def _coerce_bool_to_int(typed_expr: TypedExpr) -> sge.Expression:
+    """Coerce boolean expression to integer."""
+    if typed_expr.dtype == dtypes.BOOL_DTYPE:
+        return sge.Cast(this=typed_expr.expr, to="INT64")
+    return typed_expr.expr
+
+
+def _coerce_date_to_datetime(typed_expr: TypedExpr) -> sge.Expression:
+    """Coerce date expression to datetime."""
+    if typed_expr.dtype == dtypes.DATE_DTYPE:
+        return sge.Cast(this=typed_expr.expr, to="DATETIME")
+    return typed_expr.expr
