@@ -17,9 +17,14 @@ import benchmark.utils as utils
 import bigframes_vendored.tpch.queries.q17 as vendored_tpch_q17
 
 if __name__ == "__main__":
-    project_id, dataset_id, session, suffix = utils.get_configuration()
+    config = utils.get_configuration()
     current_path = pathlib.Path(__file__).absolute()
 
     utils.get_execution_time(
-        vendored_tpch_q17.q, current_path, suffix, project_id, dataset_id, session
+        vendored_tpch_q17.q,
+        current_path,
+        config.benchmark_suffix,
+        config.project_id,
+        config.dataset_id,
+        config.session,
     )
