@@ -107,6 +107,24 @@ def test_div_timedelta(scalar_types_df: bpd.DataFrame, snapshot):
     snapshot.assert_match(bf_df.sql, "out.sql")
 
 
+def test_eq_null_match(scalar_types_df: bpd.DataFrame, snapshot):
+    bf_df = scalar_types_df[["int64_col", "bool_col"]]
+    sql = _apply_binary_op(bf_df, ops.eq_null_match_op, "int64_col", "bool_col")
+    snapshot.assert_match(sql, "out.sql")
+
+
+def test_eq_numeric(scalar_types_df: bpd.DataFrame, snapshot):
+    bf_df = scalar_types_df[["int64_col", "bool_col"]]
+
+    bf_df["int_ne_int"] = bf_df["int64_col"] == bf_df["int64_col"]
+    bf_df["int_ne_1"] = bf_df["int64_col"] == 1
+
+    bf_df["int_ne_bool"] = bf_df["int64_col"] == bf_df["bool_col"]
+    bf_df["bool_ne_int"] = bf_df["bool_col"] == bf_df["int64_col"]
+
+    snapshot.assert_match(bf_df.sql, "out.sql")
+
+
 def test_floordiv_numeric(scalar_types_df: bpd.DataFrame, snapshot):
     bf_df = scalar_types_df[["int64_col", "bool_col", "float64_col"]]
 
@@ -121,8 +139,6 @@ def test_floordiv_numeric(scalar_types_df: bpd.DataFrame, snapshot):
     bf_df["int_div_bool"] = bf_df["int64_col"] // bf_df["bool_col"]
     bf_df["bool_div_int"] = bf_df["bool_col"] // bf_df["int64_col"]
 
-    snapshot.assert_match(bf_df.sql, "out.sql")
-
 
 def test_floordiv_timedelta(scalar_types_df: bpd.DataFrame, snapshot):
     bf_df = scalar_types_df[["timestamp_col", "date_col"]]
@@ -133,6 +149,30 @@ def test_floordiv_timedelta(scalar_types_df: bpd.DataFrame, snapshot):
     snapshot.assert_match(bf_df.sql, "out.sql")
 
 
+def test_gt_numeric(scalar_types_df: bpd.DataFrame, snapshot):
+    bf_df = scalar_types_df[["int64_col", "bool_col"]]
+
+    bf_df["int_gt_int"] = bf_df["int64_col"] > bf_df["int64_col"]
+    bf_df["int_gt_1"] = bf_df["int64_col"] > 1
+
+    bf_df["int_gt_bool"] = bf_df["int64_col"] > bf_df["bool_col"]
+    bf_df["bool_gt_int"] = bf_df["bool_col"] > bf_df["int64_col"]
+
+    snapshot.assert_match(bf_df.sql, "out.sql")
+
+
+def test_ge_numeric(scalar_types_df: bpd.DataFrame, snapshot):
+    bf_df = scalar_types_df[["int64_col", "bool_col"]]
+
+    bf_df["int_ge_int"] = bf_df["int64_col"] >= bf_df["int64_col"]
+    bf_df["int_ge_1"] = bf_df["int64_col"] >= 1
+
+    bf_df["int_ge_bool"] = bf_df["int64_col"] >= bf_df["bool_col"]
+    bf_df["bool_ge_int"] = bf_df["bool_col"] >= bf_df["int64_col"]
+
+    snapshot.assert_match(bf_df.sql, "out.sql")
+
+
 def test_json_set(json_types_df: bpd.DataFrame, snapshot):
     bf_df = json_types_df[["json_col"]]
     sql = _apply_binary_op(
@@ -142,6 +182,30 @@ def test_json_set(json_types_df: bpd.DataFrame, snapshot):
     snapshot.assert_match(sql, "out.sql")
 
 
+def test_lt_numeric(scalar_types_df: bpd.DataFrame, snapshot):
+    bf_df = scalar_types_df[["int64_col", "bool_col"]]
+
+    bf_df["int_lt_int"] = bf_df["int64_col"] < bf_df["int64_col"]
+    bf_df["int_lt_1"] = bf_df["int64_col"] < 1
+
+    bf_df["int_lt_bool"] = bf_df["int64_col"] < bf_df["bool_col"]
+    bf_df["bool_lt_int"] = bf_df["bool_col"] < bf_df["int64_col"]
+
+    snapshot.assert_match(bf_df.sql, "out.sql")
+
+
+def test_le_numeric(scalar_types_df: bpd.DataFrame, snapshot):
+    bf_df = scalar_types_df[["int64_col", "bool_col"]]
+
+    bf_df["int_le_int"] = bf_df["int64_col"] <= bf_df["int64_col"]
+    bf_df["int_le_1"] = bf_df["int64_col"] <= 1
+
+    bf_df["int_le_bool"] = bf_df["int64_col"] <= bf_df["bool_col"]
+    bf_df["bool_le_int"] = bf_df["bool_col"] <= bf_df["int64_col"]
+
+    snapshot.assert_match(bf_df.sql, "out.sql")
+
+
 def test_sub_numeric(scalar_types_df: bpd.DataFrame, snapshot):
     bf_df = scalar_types_df[["int64_col", "bool_col"]]
 
@@ -200,3 +264,15 @@ def test_mul_timedelta(scalar_types_df: bpd.DataFrame, snapshot):
 def test_obj_make_ref(scalar_types_df: bpd.DataFrame, snapshot):
     blob_df = scalar_types_df["string_col"].str.to_blob()
     snapshot.assert_match(blob_df.to_frame().sql, "out.sql")
+
+
+def test_ne_numeric(scalar_types_df: bpd.DataFrame, snapshot):
+    bf_df = scalar_types_df[["int64_col", "bool_col"]]
+
+    bf_df["int_ne_int"] = bf_df["int64_col"] != bf_df["int64_col"]
+    bf_df["int_ne_1"] = bf_df["int64_col"] != 1
+
+    bf_df["int_ne_bool"] = bf_df["int64_col"] != bf_df["bool_col"]
+    bf_df["bool_ne_int"] = bf_df["bool_col"] != bf_df["int64_col"]
+
+    snapshot.assert_match(bf_df.sql, "out.sql")
